/*
 * Pidgin TooButtons Plugin
 *
 * Copyright 2010-2011 VaYurik <vayurik@mail.ru>
 * Based on code by Etan Reisner <deryni@pidgin.im> (SendButton plugin)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* При допиливании убедительная просьба сохранить форматирование исходного кода в текущем стиле */
/* Please, save formatting of code if you want to saw this source */

#ifdef HAVE_CONFIG_H
#include "../toobuttons_config.h"
#endif

#ifndef PURPLE_PLUGINS
#define PURPLE_PLUGINS
#endif

#include "internal.h"

#include "version.h"
#include "gtkconv.h"
#include "gtkplugin.h"
#include "debug.h"
#include "gdk/gdkkeysyms.h"

#define PLUGIN_ID						"gtk-toobuttons"
#define PLUGIN_VERSION			"1.5"
#define PLUGIN_NAME					_("Send'n'Close Buttons") //
#define PLUGIN_SUMMARY			_("Adds \"Send\" and \"Close\" buttons to conversation window")
#define PLUGIN_DESCRIPTION	_("Adds \"Send\" and \"Close\" buttons to conversation window. It is convenient to use on netbooks and touchscreens.")
#define PLUGIN_AUTHOR				"VaYurik <vayurik@mail.ru>"
#define PLUGIN_WEBSITE			"http://vayurik.ru/wordpress/toobuttons"

#define IS_DEBUGGING				FALSE // Режим отображения отладочной информации

#define BUTTON_SET 					"toobuttons_buttonset"
#define MENU_SET 						"toobuttons_menuset"
#define SIGNAL_SET 					"toobuttons_signalset"

static void Debugging (const char *str)
{
	if (IS_DEBUGGING) purple_debug_info (PLUGIN_ID, "%s\n", str);
}

static void CloseConv (PidginConversation *gtkconv)
{
	Debugging ("CloseConv");
	if (gtkconv == NULL) return;
	if (purple_prefs_get_bool ("/plugins/gtk/toobuttons/close_hide")) g_signal_emit_by_name (gtkconv->close, "clicked");
	else purple_conversation_destroy (gtkconv->active_conv);
}

static void CloseAllConv (void)
{
	GList *iter = pidgin_conv_windows_get_list ();
	Debugging ("CloseAllConv");
	while (iter)
	{
		GList *it = pidgin_conv_window_get_gtkconvs (iter->data);
		iter = iter->next;
		while (it)
		{
			PidginConversation *gtkconv = it->data;
			it = it->next;
			CloseConv (gtkconv);
		}
	}
}

static void CloseAllConv_cb (GtkButton *widget, gpointer data)
{
	Debugging ("CloseAllConv_cb");
	CloseAllConv ();
}

static void SendButton_cb (GtkButton *button, PidginConversation *gtkconv)
{
	Debugging ("SendButton_cb");
	if (gtkconv == NULL) return;
	g_signal_emit_by_name (gtkconv->entry, "message_send");
	if (purple_prefs_get_bool ("/plugins/gtk/toobuttons/send_close")) 
	{
		if (purple_prefs_get_bool ("/plugins/gtk/toobuttons/close_all")) CloseAllConv ();
		else CloseConv (gtkconv);
	}
}

static void CloseButton_cb (GtkButton *button, PidginConversation *gtkconv)
{
	Debugging ("CloseButton_cb");
	if (purple_prefs_get_bool ("/plugins/gtk/toobuttons/close_all")) CloseAllConv ();
	else CloseConv (gtkconv);
}

static void SetSendButton (GtkTextBuffer *TextBuffer, GtkWidget *SendButton)
{
	Debugging ("SetSendButton");
	if (gtk_text_buffer_get_char_count (TextBuffer) != 0) gtk_widget_set_sensitive (SendButton, TRUE);
	else gtk_widget_set_sensitive (SendButton, FALSE);
}

static void CreateButtons (PidginConversation *gtkconv)
{
	GtkWidget *Box_bbox;
	GtkWidget *Send_btn, *Close_btn;
	GtkWidget *ButtonBox, *icon, *label;
	GtkWidget *tb, *parent;
	GtkTextBuffer *buf;

	Debugging ("CreateButtons");
	if (gtkconv == NULL) return;
	tb = g_object_get_data (G_OBJECT (gtkconv->lower_hbox), BUTTON_SET);
	if (tb != NULL) return;

	Box_bbox = gtk_hbutton_box_new ();
	gtk_button_box_set_layout (GTK_BUTTON_BOX (Box_bbox), GTK_BUTTONBOX_END);
	gtk_box_set_spacing (GTK_BOX (Box_bbox), PIDGIN_HIG_BOX_SPACE);
	parent = gtk_widget_get_parent (GTK_WIDGET (gtkconv->lower_hbox));
	parent = gtk_widget_get_parent (GTK_WIDGET (parent));
	gtk_box_pack_end (GTK_BOX (parent), Box_bbox, FALSE, FALSE, 0);

	ButtonBox = gtk_hbox_new (FALSE, 0);
	icon = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
	gtk_box_pack_start (GTK_BOX (ButtonBox), icon, FALSE, FALSE, 2);
	label = gtk_label_new_with_mnemonic (_("_Close"));
	gtk_box_pack_start (GTK_BOX (ButtonBox), label, FALSE, FALSE, 0);
	Close_btn = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER (Close_btn), ButtonBox);
	g_signal_connect (G_OBJECT (Close_btn), "clicked", G_CALLBACK (CloseButton_cb), gtkconv);
	gtk_box_pack_end (GTK_BOX (Box_bbox), Close_btn, FALSE, FALSE, 0);

	ButtonBox = gtk_hbox_new (FALSE, 0);
	icon = gtk_image_new_from_stock (GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON);
	gtk_box_pack_start (GTK_BOX (ButtonBox), icon, FALSE, FALSE, 2);
	label = gtk_label_new_with_mnemonic (_("_Send"));
	gtk_box_pack_start (GTK_BOX (ButtonBox), label, FALSE, FALSE, 0);
	Send_btn = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER (Send_btn), ButtonBox);
	g_signal_connect (G_OBJECT (Send_btn), "clicked", G_CALLBACK (SendButton_cb), gtkconv);
	gtk_box_pack_end (GTK_BOX (Box_bbox), Send_btn, FALSE, FALSE, 0);

	gtk_widget_show_all (Box_bbox);

	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (gtkconv->entry));
	if (buf)
	{
		guint SignalId;
		SignalId = g_signal_connect (G_OBJECT (buf), "changed", G_CALLBACK (SetSendButton), Send_btn);
		g_object_set_data (G_OBJECT (Send_btn), SIGNAL_SET, GINT_TO_POINTER (SignalId));
		SetSendButton (buf, Send_btn);
	}
	g_object_set_data (G_OBJECT (gtkconv->lower_hbox), BUTTON_SET, Box_bbox);
}

static void RemoveButtons (PidginConversation *gtkconv)
{
	GtkWidget *tb = NULL;
	Debugging ("RemoveButtons");
	if (gtkconv == NULL) return;
	tb = g_object_get_data (G_OBJECT (gtkconv->lower_hbox), BUTTON_SET);
	if (tb != NULL)
	{
		GtkTextBuffer *buf;
		guint SignalId;
		buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (gtkconv->entry));
		SignalId = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tb), SIGNAL_SET));
		if (buf && SignalId) g_signal_handler_disconnect (G_OBJECT (buf), SignalId);
		gtk_widget_destroy (tb);
		g_object_set_data (G_OBJECT (gtkconv->lower_hbox), BUTTON_SET, NULL);
	}
}

static void DrawButtons (PidginConversation *gtkconv)
{
	GtkWidget *tb = NULL;
	Debugging ("DrawButtons");
	tb = g_object_get_data (G_OBJECT (gtkconv->lower_hbox), BUTTON_SET);
	if (tb == NULL) CreateButtons (gtkconv);
}

static void TogglePref_cb (GtkWidget *widget, gpointer data)
{
	Debugging ("TogglePref_cb");
	purple_prefs_set_bool ((gchar *) data, gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)));
}

static void CreateSubMenu (PidginConversation *gtkconv)
{
	PidginWindow *win;
	GtkWidget *menu, *item, *img;
	GtkAccelGroup *accel = NULL;

	Debugging ("CreateSubMenu");
	if (gtkconv == NULL) return;
	win = pidgin_conv_get_window (gtkconv);
	if (win == NULL) return;
	if (g_object_get_data (G_OBJECT (win->window), MENU_SET)) return;
	g_object_set_data (G_OBJECT (win->window), MENU_SET, GINT_TO_POINTER (TRUE));

	menu = gtk_item_factory_get_widget (win->menu.item_factory, N_("/Conversation"));

	accel = gtk_accel_group_new ();
	gtk_window_add_accel_group (GTK_WINDOW (win->window), accel);
	item = gtk_image_menu_item_new_with_mnemonic (_("Close All"));
	img = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), img);
	gtk_widget_add_accelerator (item, "activate", accel, GDK_w, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_ACCEL_VISIBLE);
	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (CloseAllConv_cb), NULL);
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	g_object_set_data (G_OBJECT (item), MENU_SET, GINT_TO_POINTER (TRUE));

	menu = gtk_item_factory_get_widget (win->menu.item_factory, N_("/Options"));

	item = gtk_separator_menu_item_new ();
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	g_object_set_data (G_OBJECT (item), MENU_SET, GINT_TO_POINTER (TRUE));

	item = gtk_check_menu_item_new_with_mnemonic (_("Close button _hides the conversation"));
	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (item), TRUE);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), purple_prefs_get_bool ("/plugins/gtk/toobuttons/close_hide"));
	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (TogglePref_cb), "/plugins/gtk/toobuttons/close_hide");
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	g_object_set_data (G_OBJECT (item), MENU_SET, GINT_TO_POINTER (TRUE));

	if (purple_prefs_get_bool ("/plugins/gtk/toobuttons/close_hide"))
		item = gtk_check_menu_item_new_with_mnemonic (_("Close button hides _all conversations"));
	else
		item = gtk_check_menu_item_new_with_mnemonic (_("Close button closes _all conversations"));
	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (item), TRUE);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), purple_prefs_get_bool ("/plugins/gtk/toobuttons/close_all"));
	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (TogglePref_cb), "/plugins/gtk/toobuttons/close_all");
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	g_object_set_data (G_OBJECT (item), MENU_SET, GINT_TO_POINTER (TRUE));

	if (purple_prefs_get_bool ("/plugins/gtk/toobuttons/close_hide"))
		item = gtk_check_menu_item_new_with_mnemonic (_("Hide _conversation after sending message"));
	else
		item = gtk_check_menu_item_new_with_mnemonic (_("Close _conversation after sending message"));
	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (item), TRUE);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), purple_prefs_get_bool ("/plugins/gtk/toobuttons/send_close"));
	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (TogglePref_cb), "/plugins/gtk/toobuttons/send_close");
	gtk_widget_show (item);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
	g_object_set_data (G_OBJECT (item), MENU_SET, GINT_TO_POINTER (TRUE));
}

static void RemoveSubMenu (PidginConversation *gtkconv)
{
	PidginWindow *win;
	GtkWidget *widget;
	GList *children;

	Debugging ("RemoveSubMenu");
	if (gtkconv == NULL) return;
	win = pidgin_conv_get_window (gtkconv);
	if (win == NULL) return;
	widget = gtk_item_factory_get_widget (win->menu.item_factory, N_("/Conversation"));
	children = gtk_container_get_children (GTK_CONTAINER (widget));
	while (children)
	{
		GtkWidget *item = children->data;
		children = children->next;
		if (g_object_get_data (G_OBJECT(item), MENU_SET)) gtk_widget_destroy (item);
	}
	widget = gtk_item_factory_get_widget (win->menu.item_factory, N_("/Options"));
	children = gtk_container_get_children (GTK_CONTAINER (widget));
	while (children)
	{
		GtkWidget *item = children->data;
		children = children->next;
		if (g_object_get_data (G_OBJECT(item), MENU_SET)) gtk_widget_destroy (item);
	}
	g_object_set_data (G_OBJECT (win->window), MENU_SET, GINT_TO_POINTER (FALSE));
}

static void ReCreateMenu_cb (const gchar *name, PurplePrefType type, gconstpointer value, gpointer data)
{
	GList *convs = purple_get_conversations ();
	Debugging ("ReCreateMenu_cb");
	while (convs)
	{
		PurpleConversation *conv = (PurpleConversation *) convs->data;
		if (PIDGIN_IS_PIDGIN_CONVERSATION (conv))
		{
			PidginConversation *gtkconv = PIDGIN_CONVERSATION (conv);
			RemoveSubMenu (gtkconv);
			CreateSubMenu (gtkconv);
		}
		convs = convs->next;
	}
}

static void ConvDisplayed_cb (PidginConversation *gtkconv)
{
	Debugging ("ConvDisplayed_cb");
	if ((!gtkconv) || (!gtkconv->win) || (!gtkconv->win->window)) return;
	if (!GTK_WIDGET_VISIBLE (gtkconv->win->window)) return;
	CreateSubMenu (gtkconv);
	DrawButtons (gtkconv);
}

static gboolean PluginLoad (PurplePlugin *plugin)
{
	GList *convs = purple_get_conversations ();
	Debugging ("PluginLoad");
	while (convs)
	{
		PurpleConversation *conv = (PurpleConversation *) convs->data;
		if (PIDGIN_IS_PIDGIN_CONVERSATION (conv))
		{
			PidginConversation *gtkconv = PIDGIN_CONVERSATION (conv);
			CreateButtons (gtkconv);
			CreateSubMenu (gtkconv);
		}
		convs = convs->next;
	}
	purple_signal_connect (pidgin_conversations_get_handle (), "conversation-displayed", plugin, PURPLE_CALLBACK (ConvDisplayed_cb), NULL);
	purple_prefs_connect_callback (plugin, "/plugins/gtk/toobuttons/close_hide", ReCreateMenu_cb, NULL);
	purple_prefs_connect_callback (plugin, "/plugins/gtk/toobuttons/close_all", ReCreateMenu_cb, NULL);
	purple_prefs_connect_callback (plugin, "/plugins/gtk/toobuttons/send_close", ReCreateMenu_cb, NULL);
	return TRUE;
}

static gboolean PluginUnload (PurplePlugin *plugin)
{
	GList *convs = purple_get_conversations ();
	Debugging ("PluginUnload");
	while (convs)
	{
		PurpleConversation *conv = (PurpleConversation *) convs->data;
		if (PIDGIN_IS_PIDGIN_CONVERSATION (conv))
		{
			PidginConversation *gtkconv = PIDGIN_CONVERSATION (conv);
			RemoveButtons (gtkconv);
			RemoveSubMenu (gtkconv);
		}
		convs = convs->next;
	}
	purple_signals_disconnect_by_handle (plugin);
	return TRUE;
}

static PurplePluginPrefFrame *CreatePrefFrame (PurplePlugin *plugin)
{
	PurplePluginPrefFrame *frame;
	PurplePluginPref *pref;
	Debugging ("CreatePrefFrame");
	frame = purple_plugin_pref_frame_new ();
	pref = purple_plugin_pref_new_with_label (_("Close button"));
	purple_plugin_pref_frame_add (frame, pref);
	pref = purple_plugin_pref_new_with_name_and_label ("/plugins/gtk/toobuttons/close_hide", _("Close button _hides the conversation (otherwise - closes)"));
	purple_plugin_pref_frame_add (frame, pref);
	pref = purple_plugin_pref_new_with_name_and_label ("/plugins/gtk/toobuttons/close_all", _("Close button hides (or closes) _all conversations"));
	purple_plugin_pref_frame_add (frame, pref);
	pref = purple_plugin_pref_new_with_label (_("Send button"));
	purple_plugin_pref_frame_add (frame, pref);
	pref = purple_plugin_pref_new_with_name_and_label ("/plugins/gtk/toobuttons/send_close", _("Hide (or close) _conversation after sending message"));
	purple_plugin_pref_frame_add (frame, pref);
	return frame;
}

static PurplePluginUiInfo Preferences =
{
	CreatePrefFrame,
	0,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

static PurplePluginInfo info =
{
	PURPLE_PLUGIN_MAGIC,
	PURPLE_MAJOR_VERSION,
	PURPLE_MINOR_VERSION,
	PURPLE_PLUGIN_STANDARD,
	PIDGIN_PLUGIN_TYPE,
	0,
	NULL,

	PURPLE_PRIORITY_DEFAULT,
	PLUGIN_ID,
	NULL,
	PLUGIN_VERSION,
	NULL,
	NULL,
	PLUGIN_AUTHOR,
	PLUGIN_WEBSITE,

	PluginLoad,
	PluginUnload,
	NULL,

	NULL,
	NULL,
	&Preferences,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL
};

static void InitPlugin (PurplePlugin *plugin)
{
	Debugging ("InitPlugin");
#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
#endif
	info.name = PLUGIN_NAME;
	info.summary = PLUGIN_SUMMARY;
	info.description = PLUGIN_DESCRIPTION;

	purple_prefs_add_none ("/plugins/gtk/toobuttons");
	purple_prefs_add_bool ("/plugins/gtk/toobuttons/close_hide", TRUE);
	purple_prefs_add_bool ("/plugins/gtk/toobuttons/close_all", TRUE);
	purple_prefs_add_bool ("/plugins/gtk/toobuttons/send_close", FALSE);
}

PURPLE_INIT_PLUGIN (toobuttons, InitPlugin, info)
